import json
import os
import uuid
import zipfile
import tkinter as tk
from tkinter import ttk, filedialog, simpledialog, messagebox, scrolledtext
import requests 
import tempfile 
import shutil
import subprocess
import re
import threading
import sys 
import traceback

try:
    from PIL import Image, ImageTk
    PIL_AVAILABLE = True
except ImportError:
    PIL_AVAILABLE = False

# --- Constants ---
MANIFEST_FILE_NAME = "store_manifest.json"
APKS_DIR_NAME = "apks"
IMAGES_DIR_NAME = "images"
CONFIG_FILE_FOR_SCRIPT = "kosher_store_gui_config.json" # Not actively used yet for saving/loading GUI state

# --- Icon Extractor Settings & Helpers ---
PREFERRED_DENSITIES_ORDER = [
    'xxxhdpi', 'xxhdpi', 'xhdpi', 'hdpi', 'mdpi', 'ldpi', 'anydpi', 'nodpi'
]
COMMON_ICON_NAMES = [
    'ic_launcher.png', 'icon.png', 'ic_launcher_round.png', 'app_icon.png',
    'logo.png', 'appicon.png', 'launcher_icon.png',
    'ic_launcher_foreground.png', 'ic_launcher_background.png',
    'ic_launcher_foreground.webp', 'ic_launcher_background.webp'
]
IMAGE_EXTENSIONS = ('.png', '.webp', '.jpg', '.jpeg', '.gif', '.ico', '.xml')

CACHED_AAPT_PATH = None

def log_message_gui(text_area, message):
    if text_area and hasattr(text_area, 'winfo_exists') and text_area.winfo_exists():
        current_state = text_area.cget("state")
        text_area.configure(state='normal')
        text_area.insert(tk.END, str(message) + "\n")
        text_area.see(tk.END)
        if current_state == 'disabled':
            text_area.configure(state='disabled')
        text_area.update_idletasks()
    else:
        print(f"GUI_LOG (fallback): {message}", file=sys.stderr)

def find_best_icon_path_from_list(icon_paths, log_area, prefer_png_over_webp=True, specific_filename=None):
    best_icon_candidate = None
    best_density_index = float('inf')
    best_is_png_or_high_quality_webp = False

    image_icon_paths_no_xml = [p for p in icon_paths if p.lower().endswith(tuple(ext for ext in IMAGE_EXTENSIONS if ext != '.xml'))]

    if specific_filename:
        image_icon_paths_no_xml = [p for p in image_icon_paths_no_xml if os.path.basename(p.lower()) == specific_filename.lower()]
        if not image_icon_paths_no_xml:
            log_message_gui(log_area, f"    בחירת נתיב: לא נמצא הקובץ הספציפי '{specific_filename}' ברשימת המועמדים לתמונות.")
            return None

    if not image_icon_paths_no_xml:
        log_message_gui(log_area, "    בחירת נתיב: לא נמצאו נתיבי קבצי תמונה (לא XML) ברשימת המועמדים.")
        return None

    for path in image_icon_paths_no_xml:
        path_lower = path.lower()
        current_density_index = float('inf')
        found_density_in_path = False
        
        for i, density in enumerate(PREFERRED_DENSITIES_ORDER):
            if f'-{density}' in path_lower or f'/{density}/' in path_lower or \
               f'drawable-{density}' in path_lower or f'mipmap-{density}' in path_lower :
                current_density_index = i
                found_density_in_path = True
                break
        
        if not found_density_in_path:
            current_density_index = len(PREFERRED_DENSITIES_ORDER)
            if 'anydpi' in path_lower:
                 current_density_index = PREFERRED_DENSITIES_ORDER.index('anydpi') if 'anydpi' in PREFERRED_DENSITIES_ORDER else current_density_index
            elif 'nodpi' in path_lower:
                 current_density_index = PREFERRED_DENSITIES_ORDER.index('nodpi') if 'nodpi' in PREFERRED_DENSITIES_ORDER else current_density_index

        is_png = path_lower.endswith('.png')
        is_webp = path_lower.endswith('.webp')

        if current_density_index < best_density_index:
            best_density_index = current_density_index
            best_icon_candidate = path
            best_is_png_or_high_quality_webp = is_png or is_webp
        elif current_density_index == best_density_index:
            current_is_preferred = False
            if is_png and not best_is_png_or_high_quality_webp: current_is_preferred = True
            elif is_png and best_is_png_or_high_quality_webp and best_icon_candidate and not best_icon_candidate.lower().endswith('.png'): current_is_preferred = True
            elif is_webp and not best_is_png_or_high_quality_webp and best_icon_candidate and not best_icon_candidate.lower().endswith(('.png', '.webp')) : current_is_preferred = True
            if current_is_preferred:
                best_icon_candidate = path
                best_is_png_or_high_quality_webp = is_png or is_webp
            elif 'ic_launcher.png' in path_lower and best_icon_candidate and 'ic_launcher_round.png' in best_icon_candidate.lower():
                best_icon_candidate = path
                best_is_png_or_high_quality_webp = is_png or is_webp
    
    if best_icon_candidate and not specific_filename:
        log_message_gui(log_area, f"    בחירת נתיב: המועמד שנבחר הוא '{best_icon_candidate}' (איכות DPI אינדקס {best_density_index}).")
    elif not best_icon_candidate and not specific_filename:
        log_message_gui(log_area, "    בחירת נתיב: לא נבחר אייקון תמונה מבין המועמדים.")
        
    return best_icon_candidate

def get_aapt_path(log_area_for_startup=None):
    global CACHED_AAPT_PATH
    if CACHED_AAPT_PATH and os.path.exists(CACHED_AAPT_PATH):
        if log_area_for_startup: log_message_gui(log_area_for_startup, f"  בדיקת aapt: משתמש בנתיב שמור: {CACHED_AAPT_PATH}")
        return CACHED_AAPT_PATH
    aapt_exe_name = "aapt.exe" if os.name == 'nt' else "aapt"; aapt_paths_to_try = [aapt_exe_name]
    sdk_env_vars = ['ANDROID_HOME', 'ANDROID_SDK_ROOT']; sdk_roots = [os.environ.get(var) for var in sdk_env_vars if os.environ.get(var)]
    common_sdk_paths_windows = [os.path.join(os.environ.get('LOCALAPPDATA', ''), 'Android', 'Sdk'), 'C:\\Android\\sdk']
    common_sdk_paths_linux_mac = [os.path.join(os.environ.get('HOME', ''), 'Android/Sdk'), os.path.join(os.environ.get('HOME', ''), 'android-sdk'), '/usr/lib/android-sdk', '/opt/android-sdk']
    if os.name == 'nt':
        sdk_roots.extend(common_sdk_paths_windows)
    else:
        sdk_roots.extend(common_sdk_paths_linux_mac)
    for sdk_root in sdk_roots:
        if sdk_root and os.path.isdir(sdk_root):
            build_tools_dir = os.path.join(sdk_root, "build-tools")
            if os.path.isdir(build_tools_dir):
                versions = []; 
                for d in os.listdir(build_tools_dir):
                    if os.path.isdir(os.path.join(build_tools_dir, d)):
                        try: parts = tuple(map(int, d.split('.'))); versions.append((parts, d))
                        except ValueError: pass 
                versions.sort(key=lambda v: v[0], reverse=True)
                if versions:
                    latest_version_str = versions[0][1] 
                    potential_aapt_path = os.path.join(build_tools_dir, latest_version_str, aapt_exe_name)
                    if potential_aapt_path not in aapt_paths_to_try: aapt_paths_to_try.append(potential_aapt_path)
    for path_try in aapt_paths_to_try:
        try:
            resolved_path = shutil.which(path_try)
            if resolved_path and os.path.exists(resolved_path) and os.access(resolved_path, os.X_OK):
                startupinfo = None; 
                if os.name == 'nt': startupinfo = subprocess.STARTUPINFO(); startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW; startupinfo.wShowWindow = subprocess.SW_HIDE
                process = subprocess.run([resolved_path, "version"], capture_output=True, text=True, startupinfo=startupinfo, timeout=3, encoding='utf-8', errors='ignore')
                if process.returncode == 0 and "Android Asset Packaging Tool" in process.stdout:
                    if log_area_for_startup: log_message_gui(log_area_for_startup, f"  בדיקת aapt: נמצא בנתיב מאומת: {resolved_path}")
                    CACHED_AAPT_PATH = resolved_path; return resolved_path
        except FileNotFoundError: 
            if log_area_for_startup and path_try == aapt_exe_name: log_message_gui(log_area_for_startup, f"  בדיקת aapt: '{path_try}' לא נמצא ב-PATH.")
            continue
        except subprocess.TimeoutExpired:
            if log_area_for_startup: log_message_gui(log_area_for_startup, f"  בדיקת aapt: פקודת 'aapt version' נתקעה עבור '{path_try}'.")
            continue
        except Exception as e:
            if log_area_for_startup: log_message_gui(log_area_for_startup, f"  בדיקת aapt: שגיאה בבדיקת '{path_try}': {e}")
            continue
    if log_area_for_startup: log_message_gui(log_area_for_startup, "  בדיקת aapt: הפקודה 'aapt' לא אותרה באופן סופי.")
    return None

def extract_icon_using_aapt(apk_path, output_dir, log_area):
    aapt_cmd = get_aapt_path(log_area)
    if not aapt_cmd:
        log_message_gui(log_area, "  חילוץ aapt: 'aapt' לא נמצא. מנסה חילוץ ידני.")
        return False, None
    log_message_gui(log_area, f"  חילוץ aapt: משתמש ב-{aapt_cmd} עבור {os.path.basename(apk_path)}...")
    try:
        startupinfo = None
        if os.name == 'nt':
            startupinfo = subprocess.STARTUPINFO(); startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW; startupinfo.wShowWindow = subprocess.SW_HIDE
        process = subprocess.run([aapt_cmd, 'dump', 'badging', apk_path], capture_output=True, text=True, encoding='utf-8', errors='ignore', startupinfo=startupinfo, timeout=10)
        if process.returncode != 0:
            log_message_gui(log_area, f"  שגיאה: aapt נכשל (קוד {process.returncode}). stderr: {process.stderr.strip()[:200]}...")
            return False, None
        badging_output = process.stdout; apk_basename_no_ext = os.path.splitext(os.path.basename(apk_path))[0]
        aapt_direct_image_icons = []; direct_matches = re.findall(r"application-icon(?:-\d+)?:\s*'([^']+)'", badging_output); aapt_direct_image_icons.extend(direct_matches)
        main_icon_match = re.search(r"application(?:-\w+)*:\s*label='(?:[^']*)'\s*icon='([^']+)'", badging_output)
        if main_icon_match:
            declared_main_icon = main_icon_match.group(1)
            if declared_main_icon not in aapt_direct_image_icons: aapt_direct_image_icons.append(declared_main_icon)
        aapt_direct_image_icons_filtered = [p for p in list(set(aapt_direct_image_icons)) if p.lower().endswith(tuple(ext for ext in IMAGE_EXTENSIONS if ext != '.xml'))]
        if aapt_direct_image_icons_filtered:
            log_message_gui(log_area, f"    aapt דיווח אייקוני תמונה ישירים: {aapt_direct_image_icons_filtered}")
            icon_to_extract = find_best_icon_path_from_list(aapt_direct_image_icons_filtered, log_area)
            if icon_to_extract:
                with zipfile.ZipFile(apk_path, 'r') as zip_ref:
                    if icon_to_extract not in zip_ref.namelist(): log_message_gui(log_area, f"    שגיאה: aapt דיווח על '{icon_to_extract}' אך הוא לא נמצא ב-APK."); return False, None
                    ext = os.path.splitext(icon_to_extract)[1].lower(); base = f"{apk_basename_no_ext}_icon_aapt"; name = f"{base}{ext}"; c=1; final_path = os.path.join(output_dir,name)
                    while os.path.exists(final_path): name=f"{base}_{c}{ext}";final_path=os.path.join(output_dir,name);c+=1
                    with open(final_path,'wb') as f_out: f_out.write(zip_ref.read(icon_to_extract))
                    log_message_gui(log_area, f"  הצלחה (aapt-ישיר): חולץ ל- {final_path}"); return True, final_path
        adaptive_xml_path = None; adaptive_match = re.search(r"application(?:-\w+)*:\s*adaptive-icon='([^']+)'", badging_output)
        if adaptive_match: adaptive_xml_path = adaptive_match.group(1)
        elif main_icon_match and main_icon_match.group(1).lower().endswith('.xml'): adaptive_xml_path = main_icon_match.group(1) # if main icon is xml
        if adaptive_xml_path and adaptive_xml_path.lower().endswith(".xml"):
            log_message_gui(log_area, f"    aapt דיווח adaptive-icon XML: '{adaptive_xml_path}'")
            xml_base = os.path.splitext(os.path.basename(adaptive_xml_path))[0]
            extracted_components = {}
            with zipfile.ZipFile(apk_path, 'r') as zf:
                all_files = zf.namelist()
                for comp_type in ["foreground", "background"]:
                    comp_names_to_try = [f"{xml_base}_{comp_type}.png", f"{xml_base}_{comp_type}.webp", f"{xml_base}.png", f"{xml_base}.webp"] # Fallback to base name
                    if comp_type == "foreground" and "ic_launcher_foreground" in COMMON_ICON_NAMES: comp_names_to_try.extend([n for n in COMMON_ICON_NAMES if "foreground" in n])
                    elif comp_type == "background" and "ic_launcher_background" in COMMON_ICON_NAMES: comp_names_to_try.extend([n for n in COMMON_ICON_NAMES if "background" in n])
                    
                    candidates = [f for f in all_files if (f.startswith("res/drawable") or f.startswith("res/mipmap")) and os.path.basename(f.lower()) in [n.lower() for n in comp_names_to_try] and f.lower().endswith(tuple(ext for ext in IMAGE_EXTENSIONS if ext != '.xml'))]
                    if candidates:
                        best_comp = find_best_icon_path_from_list(candidates, log_area, specific_filename=os.path.basename(candidates[0]))
                        if not best_comp and len(candidates) > 1: best_comp = find_best_icon_path_from_list(candidates, log_area)
                        if best_comp:
                            ext = os.path.splitext(best_comp)[1]; base = f"{apk_basename_no_ext}_adaptive_{comp_type}"; name = f"{base}{ext}"; c=1; final_path = os.path.join(output_dir,name)
                            while os.path.exists(final_path): name=f"{base}_{c}{ext}";final_path=os.path.join(output_dir,name);c+=1
                            with open(final_path,'wb') as f_out: f_out.write(zf.read(best_comp))
                            log_message_gui(log_area, f"  הצלחה (aapt-adaptive): רכיב '{comp_type}' חולץ ל- {final_path}"); extracted_components[comp_type] = final_path
            if "foreground" in extracted_components: return True, extracted_components["foreground"] # Prefer foreground
            if extracted_components: return True, list(extracted_components.values())[0] # Or any component if foreground not found
        log_message_gui(log_area, "  חילוץ aapt: לא נמצא אייקון."); return False, None
    except subprocess.TimeoutExpired: log_message_gui(log_area, f"  שגיאה: aapt נתקע עבור {os.path.basename(apk_path)}."); return False, None
    except Exception as e: log_message_gui(log_area, f"  שגיאה ב-aapt: {type(e).__name__} - {e}"); return False, None

def extract_icon_manually(apk_path, output_dir, log_area):
    log_message_gui(log_area, f"  חילוץ ידני: עבור {os.path.basename(apk_path)}...")
    try:
        with zipfile.ZipFile(apk_path, 'r') as zip_ref:
            namelist = zip_ref.namelist(); candidate_icon_paths = [];
            for item_path in namelist:
                item_path_lower = item_path.lower()
                if (item_path_lower.startswith('res/drawable') or item_path_lower.startswith('res/mipmap')) and \
                   any(name in item_path_lower for name in COMMON_ICON_NAMES) and \
                   item_path_lower.endswith(tuple(ext for ext in IMAGE_EXTENSIONS if ext != '.xml')):
                    candidate_icon_paths.append(item_path)
            if not candidate_icon_paths:
                 for item_path in namelist: # Fallback: search in root or common folders
                    item_path_lower = item_path.lower()
                    if any(name == os.path.basename(item_path_lower) for name in COMMON_ICON_NAMES) and \
                       item_path_lower.endswith(tuple(ext for ext in IMAGE_EXTENSIONS if ext != '.xml')):
                        candidate_icon_paths.append(item_path)
            if not candidate_icon_paths: log_message_gui(log_area, "  חילוץ ידני: לא נמצאו קבצי אייקון תמונה."); return False, None
            icon_to_extract = find_best_icon_path_from_list(list(set(candidate_icon_paths)), log_area)
            if not icon_to_extract: log_message_gui(log_area, "  חילוץ ידני: לא נבחר אייקון תמונה סופי."); return False, None
            apk_bn = os.path.splitext(os.path.basename(apk_path))[0]; ext = os.path.splitext(icon_to_extract)[1]; base = f"{apk_bn}_icon_manual"; name = f"{base}{ext}"; c=1; final_p = os.path.join(output_dir,name)
            while os.path.exists(final_p): name=f"{base}_{c}{ext}";final_p=os.path.join(output_dir,name);c+=1
            with open(final_p,'wb') as f_out: f_out.write(zip_ref.read(icon_to_extract))
            log_message_gui(log_area, f"  הצלחה (ידני): חולץ ל- {final_p}"); return True, final_p
    except zipfile.BadZipFile: log_message_gui(log_area, f"  שגיאה (ידני): {os.path.basename(apk_path)} אינו APK תקין."); return False, None
    except Exception as e: log_message_gui(log_area, f"  שגיאה בחילוץ ידני: {e}"); return False, None

# --- AppEntry Class ---
class AppEntry:
    def __init__(self, apk_path_or_url, is_url=False, log_area=None, 
                 output_dir_for_icons=None, initial_app_name=None, 
                 initial_pkg_name=None, initial_desc=None, initial_image_path=None):
        self.is_url = is_url
        self.apk_source = apk_path_or_url
        self.log_area = log_area
        self.output_dir_for_icons = output_dir_for_icons if output_dir_for_icons else tempfile.gettempdir()

        if is_url:
            try: self.apk_file_name = os.path.basename(requests.utils.urlparse(apk_path_or_url).path)
            except: self.apk_file_name = f"url_app_{uuid.uuid4().hex[:6]}.apk"
            if not self.apk_file_name or self.apk_file_name == '/': self.apk_file_name = f"url_app_{uuid.uuid4().hex[:6]}.apk"
        else:
            self.apk_file_name = os.path.basename(apk_path_or_url)

        self.app_name_var = tk.StringVar(value=initial_app_name if initial_app_name else os.path.splitext(self.apk_file_name)[0])
        self.package_name_var = tk.StringVar(value=initial_pkg_name if initial_pkg_name else self._generate_package_name())
        self.description_var = tk.StringVar(value=initial_desc if initial_desc else f"תיאור עבור {self.app_name_var.get()}")
        
        self.local_image_path_var = tk.StringVar(value="") # Holds full path to an icon file (either selected by user or extracted)
        self.image_file_name_in_zip_var = tk.StringVar(value="") # Just the basename for the ZIP
        
        self.file_size_var = tk.StringVar(value="0 B")
        
        if initial_image_path and os.path.exists(initial_image_path):
            self.update_image_selection(initial_image_path)
        elif not self.is_url and self.apk_source: # If local APK and no initial_image_path, try to auto-extract
             self._auto_extract_and_set_icon()

    def _generate_package_name(self):
        safe_name = ''.join(c for c in self.app_name_var.get().lower() if c.isalnum() or c=='_').replace(' ','_')
        if not safe_name: safe_name = "app"
        return f"com.generated.{safe_name}.{uuid.uuid4().hex[:4]}"

    def _auto_extract_and_set_icon(self):
        if not self.apk_source or self.is_url: return
        log_message_gui(self.log_area, f"מנסה לחלץ אייקון אוטומטית עבור: {self.apk_file_name}")
        apk_basename_no_ext = os.path.splitext(self.apk_file_name)[0]
        icon_specific_output_dir = os.path.join(self.output_dir_for_icons, apk_basename_no_ext)
        os.makedirs(icon_specific_output_dir, exist_ok=True)
        success_aapt, result_aapt = extract_icon_using_aapt(self.apk_source, icon_specific_output_dir, self.log_area)
        if success_aapt and isinstance(result_aapt, str) and os.path.exists(result_aapt):
            self.update_image_selection(result_aapt); return
        if success_aapt and result_aapt == "Adaptive components extracted": # Check if any foreground was extracted
             possible_fg = find_best_icon_path_from_list([os.path.join(icon_specific_output_dir, f) for f in os.listdir(icon_specific_output_dir) if "foreground" in f.lower()], self.log_area)
             if possible_fg and os.path.exists(possible_fg): self.update_image_selection(possible_fg); return
        success_manual, result_manual = extract_icon_manually(self.apk_source, icon_specific_output_dir, self.log_area)
        if success_manual and result_manual and os.path.exists(result_manual): self.update_image_selection(result_manual)
        else: log_message_gui(self.log_area, f"חילוץ אייקון אוטומטי נכשל עבור {self.apk_file_name}")

    def to_dict(self):
        size_str = self.file_size_var.get(); size_bytes = 0
        try:
            num_part = "".join(filter(lambda x: x.isdigit() or x == '.', size_str))
            if num_part:
                num_val = float(num_part)
                if "MB" in size_str.upper(): size_bytes = int(num_val * 1024 * 1024)
                elif "KB" in size_str.upper(): size_bytes = int(num_val * 1024)
                elif "B" in size_str.upper() and "KB" not in size_str.upper() and "MB" not in size_str.upper(): size_bytes = int(num_val)
        except ValueError: size_bytes = 0
        return {"name": self.app_name_var.get(), "description": self.description_var.get(), "apkFileName": self.apk_file_name, "imageFileName": self.image_file_name_in_zip_var.get() if self.image_file_name_in_zip_var.get() else None, "packageName": self.package_name_var.get(), "apkUrl": self.apk_source if self.is_url else None, "fileSize": size_bytes}

    def update_image_selection(self, image_path):
        self.local_image_path_var.set(image_path if image_path and os.path.exists(image_path) else "")
        self.image_file_name_in_zip_var.set(os.path.basename(image_path) if image_path and os.path.exists(image_path) else "")
    
    def get_display_name(self):
        return f"{self.app_name_var.get()} ({self.apk_file_name})"


# --- Main Application Class ---
class BackupCreatorApp:
    def __init__(self, root_window):
        self.root = root_window; self.root.title("Kosher App Store - Backup & Config Tool"); self.root.geometry("950x800")
        self.selected_apks_data = []
        self.current_loaded_zip_path = None
        self.temp_extraction_dir = tempfile.mkdtemp(prefix="kosher_backup_tool_")
        self.temp_icon_output_dir = os.path.join(self.temp_extraction_dir, "extracted_icons_temp")
        os.makedirs(self.temp_icon_output_dir, exist_ok=True)
        log_message_gui(None, f"Temporary extraction dir: {self.temp_extraction_dir}") # Log before log_area is ready

        menubar = tk.Menu(root_window); filemenu = tk.Menu(menubar, tearoff=0)
        filemenu.add_command(label="New Config", command=self.new_configuration); filemenu.add_command(label="Open Backup ZIP...", command=self.load_backup_zip_handler); filemenu.add_command(label="Save Backup ZIP", command=self.create_backup_zip_handler); filemenu.add_command(label="Save Backup ZIP As...", command=lambda: self.create_backup_zip_handler(save_as=True)); filemenu.add_separator(); filemenu.add_command(label="Exit", command=self.on_closing)
        menubar.add_cascade(label="File", menu=filemenu); root_window.config(menu=menubar)
        main_paned_window = ttk.PanedWindow(root_window, orient=tk.VERTICAL); main_paned_window.pack(fill=tk.BOTH, expand=True, padx=5, pady=5)
        settings_pane_container = ttk.Frame(main_paned_window); main_paned_window.add(settings_pane_container, weight=1)
        self.notebook = ttk.Notebook(settings_pane_container); self.notebook.pack(fill=tk.BOTH, expand=True, padx=5, pady=5)
        self.general_settings_tab = ttk.Frame(self.notebook, padding="10"); self.store_apps_tab = ttk.Frame(self.notebook, padding="10")
        self.notebook.add(self.general_settings_tab, text='General Store Settings'); self.notebook.add(self.store_apps_tab, text='Manage Apps in Store')
        self._create_settings_widgets(self.general_settings_tab)
        apps_management_pane = ttk.PanedWindow(self.store_apps_tab, orient=tk.HORIZONTAL); apps_management_pane.pack(fill=tk.BOTH, expand=True)
        apks_list_frame = ttk.LabelFrame(apps_management_pane, text="APKs / URLs in Store", padding="10"); apps_management_pane.add(apks_list_frame, weight=2); self._create_apk_list_widgets(apks_list_frame)
        self.selected_apk_details_frame = ttk.LabelFrame(apps_management_pane, text="Edit Selected App Details", padding="10"); apps_management_pane.add(self.selected_apk_details_frame, weight=3); self._create_apk_details_widgets(self.selected_apk_details_frame)
        log_frame = ttk.LabelFrame(root_window, text="Log Messages", padding="5"); log_frame.pack(fill="x", padx=5, pady=(0,5), side=tk.BOTTOM, ipady=50, anchor='s')
        self.log_area = scrolledtext.ScrolledText(log_frame, wrap=tk.WORD, height=6, state='disabled', font=("Consolas", 9)); self.log_area.pack(fill=tk.BOTH, expand=True)
        self.clear_apk_details_fields(); self._check_aapt_on_startup_gui()
        if not PIL_AVAILABLE: log_message_gui(self.log_area, "Warning: Pillow (PIL) library not found. Image previews will not work for most formats."); messagebox.showwarning("Missing Library", "Pillow (PIL) library not found. Image previews will be limited. Please install it using: pip install Pillow", parent=self.root)
        self.root.protocol("WM_DELETE_WINDOW", self.on_closing)

    def _create_settings_widgets(self, parent_frame):
        self.password_var=tk.StringVar(value="yossi6547");self.normal_install_var=tk.BooleanVar(value=False);self.use_root_var=tk.BooleanVar(value=True);self.root_method_var=tk.StringVar(value="pm_install");self.install_location_var=tk.StringVar(value="system/app");self.use_custom_storage_var=tk.BooleanVar(value=False);self.custom_storage_path_var=tk.StringVar(value="");self.send_feedback_var=tk.BooleanVar(value=True)
        r=0;ttk.Label(parent_frame,text="Store Password:").grid(row=r,column=0,sticky="w",padx=5,pady=2);ttk.Entry(parent_frame,textvariable=self.password_var,width=40).grid(row=r,column=1,sticky="ew",padx=5,pady=2);r+=1
        ttk.Checkbutton(parent_frame,text="Default: Normal Install Mode",variable=self.normal_install_var).grid(row=r,column=0,columnspan=2,sticky="w",padx=5,pady=2);r+=1
        ttk.Checkbutton(parent_frame,text="Default: Use Root Install",variable=self.use_root_var).grid(row=r,column=0,columnspan=2,sticky="w",padx=5,pady=2);r+=1
        ttk.Label(parent_frame,text="Default Root Method:").grid(row=r,column=0,sticky="w",padx=5,pady=2);ttk.Radiobutton(parent_frame,text="PM Install",variable=self.root_method_var,value="pm_install").grid(row=r,column=1,sticky="w",padx=(20,5),pady=0);r+=1;ttk.Radiobutton(parent_frame,text="Copy to System",variable=self.root_method_var,value="copy_to_system").grid(row=r,column=1,sticky="w",padx=(20,5),pady=0);r+=1
        ttk.Label(parent_frame,text="Default Copy Location:").grid(row=r,column=0,sticky="w",padx=5,pady=2);ttk.Radiobutton(parent_frame,text="/system/app",variable=self.install_location_var,value="system/app").grid(row=r,column=1,sticky="w",padx=(20,5),pady=0);r+=1;ttk.Radiobutton(parent_frame,text="/system/priv-app",variable=self.install_location_var,value="system/priv-app").grid(row=r,column=1,sticky="w",padx=(20,5),pady=0);r+=1
        ttk.Checkbutton(parent_frame,text="Use Custom Storage Path for Apps",variable=self.use_custom_storage_var,command=self.toggle_custom_path_entry).grid(row=r,column=0,columnspan=2,sticky="w",padx=5,pady=2);r+=1
        ttk.Label(parent_frame,text="Custom Apps Path:").grid(row=r,column=0,sticky="w",padx=5,pady=2);self.custom_storage_path_entry=ttk.Entry(parent_frame,textvariable=self.custom_storage_path_var,width=40,state=tk.DISABLED);self.custom_storage_path_entry.grid(row=r,column=1,sticky="ew",padx=5,pady=2);r+=1
        ttk.Checkbutton(parent_frame,text="Enable 'Send Feedback' in Menu",variable=self.send_feedback_var).grid(row=r,column=0,columnspan=2,sticky="w",padx=5,pady=2);r+=1
        ttk.Label(parent_frame,text="About Text:").grid(row=r,column=0,sticky="nw",padx=5,pady=2);self.about_text_widget=tk.Text(parent_frame,height=4,width=30,wrap=tk.WORD);self.about_text_widget.grid(row=r,column=1,sticky="nsew",padx=5,pady=2);self.about_text_widget.insert(tk.END,"Default about text for the store.");parent_frame.columnconfigure(1,weight=1);parent_frame.rowconfigure(r,weight=1)

    def toggle_custom_path_entry(self): self.custom_storage_path_entry.config(state=tk.NORMAL if self.use_custom_storage_var.get() else tk.DISABLED)

    def _create_apk_list_widgets(self, parent_frame):
        self.apk_listbox=tk.Listbox(parent_frame,selectmode=tk.SINGLE,height=10,exportselection=False);self.apk_listbox.pack(side=tk.TOP,fill="both",expand=True,pady=(0,5));self.apk_listbox.bind("<<ListboxSelect>>",self.on_apk_select_from_list)
        apk_scrollbar=ttk.Scrollbar(parent_frame,orient="vertical",command=self.apk_listbox.yview);apk_scrollbar.pack(side=tk.RIGHT,fill="y",before=self.apk_listbox);self.apk_listbox.config(yscrollcommand=apk_scrollbar.set)
        buttons_frame=ttk.Frame(parent_frame);buttons_frame.pack(fill="x",pady=5)
        ttk.Button(buttons_frame,text="Add Local APK(s)",command=self.add_local_apks_handler).pack(side=tk.LEFT,padx=2)
        ttk.Button(buttons_frame,text="Add APK from URL",command=self.add_url_apk_handler).pack(side=tk.LEFT,padx=2)
        ttk.Button(buttons_frame,text="Import Multiple (APKs+Icons Folders)",command=self.import_multiple_apks_with_icons).pack(side=tk.LEFT,padx=2)
        ttk.Button(buttons_frame,text="Remove Selected",command=self.remove_selected_apk_handler).pack(side=tk.LEFT,padx=2)

    def _create_apk_details_widgets(self, parent_frame):
        self.selected_apk_source_label=ttk.Label(parent_frame,text="Source: N/A",wraplength=350);self.selected_apk_source_label.pack(anchor="w",pady=2,fill="x")
        ipf=ttk.Frame(parent_frame);ipf.pack(anchor="w",fill="x",pady=(5,0));ttk.Label(ipf,text="Icon:").pack(side=tk.LEFT,padx=(0,5));self.icon_image_label=ttk.Label(ipf);self.icon_image_label.pack(side=tk.LEFT)
        ttk.Label(parent_frame,text="App Name (editable):").pack(anchor="w",pady=(5,0));self.selected_apk_appname_entry=ttk.Entry(parent_frame,width=45);self.selected_apk_appname_entry.pack(anchor="w",fill="x",pady=2);self.selected_apk_appname_entry.bind("<KeyRelease>",self.update_selected_apk_details_from_fields)
        ttk.Label(parent_frame,text="Package Name (editable):").pack(anchor="w",pady=(5,0));self.selected_apk_package_entry=ttk.Entry(parent_frame,width=45);self.selected_apk_package_entry.pack(anchor="w",fill="x",pady=2);self.selected_apk_package_entry.bind("<KeyRelease>",self.update_selected_apk_details_from_fields)
        ttk.Label(parent_frame,text="Description:").pack(anchor="w",pady=(5,0));self.selected_apk_description_text=tk.Text(parent_frame,height=5,width=45,wrap=tk.WORD);self.selected_apk_description_text.pack(anchor="w",fill="both",pady=2,expand=True);self.selected_apk_description_text.bind("<KeyRelease>",self.update_selected_apk_details_from_fields)
        img_f=ttk.Frame(parent_frame);img_f.pack(anchor="w",fill="x",pady=(5,0));self.selected_apk_image_label_var=tk.StringVar(value="Selected Image: None");ttk.Label(img_f,textvariable=self.selected_apk_image_label_var).pack(side=tk.LEFT,pady=2,padx=(0,5));ttk.Button(img_f,text="Override/Select Image...",command=self.select_image_for_selected_apk).pack(side=tk.LEFT,pady=2)
        size_f=ttk.Frame(parent_frame);size_f.pack(anchor="w",fill="x",pady=(5,0));self.selected_apk_size_label_var=tk.StringVar(value="Size: N/A");ttk.Label(size_f,textvariable=self.selected_apk_size_label_var).pack(side=tk.LEFT,pady=2,padx=(0,5));ttk.Button(size_f,text="Calc. Size (from URL)",command=self.calculate_apk_size_handler).pack(side=tk.LEFT,pady=2)

    def _create_action_buttons(self, parent_frame): ttk.Button(parent_frame, text="Save/Update Current Backup ZIP", command=self.create_backup_zip_handler, style="Accent.TButton").pack(pady=10, side=tk.RIGHT, padx=10); style=ttk.Style();style.configure("Accent.TButton",font=("Segoe UI",10,"bold"))
    def _check_aapt_on_startup_gui(self):threading.Thread(target=lambda:get_aapt_path(self.log_area),daemon=True).start()
    def add_local_apks_handler(self):
        files = filedialog.askopenfilenames(title="Select Local APK files",filetypes=(("APK files","*.apk"),));
        if files:
            for f_path in files:
                if not any(e.apk_source==f_path for e in self.selected_apks_data):
                    app_entry=AppEntry(f_path,is_url=False,log_area=self.log_area,output_dir_for_icons=self.temp_icon_output_dir)
                    self.selected_apks_data.append(app_entry);self.apk_listbox.insert(tk.END, app_entry.get_display_name())
            if self.selected_apks_data and self.apk_listbox.size()>0:self.apk_listbox.selection_clear(0,tk.END);self.apk_listbox.selection_set(self.apk_listbox.size()-1);self.on_apk_select_from_list(None)
    def add_url_apk_handler(self):
        url=simpledialog.askstring("Add APK from URL","Enter APK URL:",parent=self.root)
        if url:
            if not any(e.apk_source==url for e in self.selected_apks_data):
                app_entry=AppEntry(url,is_url=True,log_area=self.log_area,output_dir_for_icons=self.temp_icon_output_dir)
                self.selected_apks_data.append(app_entry);self.apk_listbox.insert(tk.END,app_entry.get_display_name())
            if self.selected_apks_data and self.apk_listbox.size()>0:self.apk_listbox.selection_clear(0,tk.END);self.apk_listbox.selection_set(self.apk_listbox.size()-1);self.on_apk_select_from_list(None)
    def import_multiple_apks_with_icons(self):
        messagebox.showinfo("Import Multiple","Select APKs folder.",parent=self.root);apk_dir=filedialog.askdirectory(title="Select APKs Folder");
        if not apk_dir: return
        messagebox.showinfo("Import Multiple","Select Icons folder.",parent=self.root);icon_dir=filedialog.askdirectory(title="Select Icons Folder")
        if not icon_dir: return
        log_message_gui(self.log_area,f"\nImporting from APKs: {apk_dir}, Icons: {icon_dir}")
        added_count=0
        for apk_fn in os.listdir(apk_dir):
            if apk_fn.lower().endswith(".apk"):
                apk_base=os.path.splitext(apk_fn)[0];apk_path=os.path.join(apk_dir,apk_fn)
                if any(e.apk_source==apk_path for e in self.selected_apks_data):continue
                icon_path=None
                for img_ext in IMAGE_EXTENSIONS:
                    pot_icon=os.path.join(icon_dir,f"{apk_base}{img_ext}")
                    if os.path.exists(pot_icon):icon_path=pot_icon;break
                app_entry=AppEntry(apk_path,is_url=False,log_area=self.log_area,output_dir_for_icons=self.temp_icon_output_dir,initial_image_path=icon_path)
                self.selected_apks_data.append(app_entry);self.apk_listbox.insert(tk.END,app_entry.get_display_name());added_count+=1
        log_message_gui(self.log_area,f"Finished. Added {added_count} apps.")
        if self.selected_apks_data and self.apk_listbox.size()>0:self.apk_listbox.selection_clear(0,tk.END);self.apk_listbox.selection_set(self.apk_listbox.size()-1);self.on_apk_select_from_list(None)
    def remove_selected_apk_handler(self):
        sel=self.apk_listbox.curselection();
        if sel:idx=sel[0];self.apk_listbox.delete(idx);del self.selected_apks_data[idx]
        if self.apk_listbox.size()>0:new_sel=max(0,idx-1) if idx==self.apk_listbox.size() else idx;self.apk_listbox.selection_set(new_sel);self.on_apk_select_from_list(None)
        else:self.clear_apk_details_fields()
    def on_apk_select_from_list(self,event):
        sel=self.apk_listbox.curselection()
        if sel:
            idx=sel[0];app=self.selected_apks_data[idx]
            self.selected_apk_source_label.config(text=f"Source: {app.apk_source}")
            for entry,var_attr in [(self.selected_apk_appname_entry,app.app_name_var),(self.selected_apk_package_entry,app.package_name_var)]:entry.config(state=tk.NORMAL);entry.delete(0,tk.END);entry.insert(0,var_attr.get())
            self.selected_apk_description_text.config(state=tk.NORMAL);self.selected_apk_description_text.delete("1.0",tk.END);self.selected_apk_description_text.insert("1.0",app.description_var.get())
            self.selected_apk_image_label_var.set(f"Image: {os.path.basename(app.local_image_path_var.get()) if app.local_image_path_var.get() else 'None'}")
            self.display_icon_preview(app.local_image_path_var.get())
            self.selected_apk_size_label_var.set(f"Size: {app.file_size_var.get()}")
        else:self.clear_apk_details_fields()
    def display_icon_preview(self,image_path):
        if PIL_AVAILABLE and image_path and os.path.exists(image_path):
            try:img=Image.open(image_path);img.thumbnail((64,64));photo=ImageTk.PhotoImage(img);self.icon_image_label.config(image=photo);self.icon_image_label.image=photo
            except Exception as e:log_message_gui(self.log_area,f"Icon preview error: {e}");self.icon_image_label.config(image='')
        else:self.icon_image_label.config(image='')
    def update_selected_apk_details_from_fields(self,event=None):
        sel=self.apk_listbox.curselection()
        if sel:
            idx=sel[0];entry=self.selected_apks_data[idx];old_disp=self.apk_listbox.get(idx)
            entry.app_name_var.set(self.selected_apk_appname_entry.get());entry.package_name_var.set(self.selected_apk_package_entry.get());entry.description_var.set(self.selected_apk_description_text.get("1.0",tk.END).strip())
            new_disp=entry.get_display_name();
            if old_disp!=new_disp:self.apk_listbox.delete(idx);self.apk_listbox.insert(idx,new_disp);self.apk_listbox.selection_set(idx)
    def select_image_for_selected_apk(self):
        sel=self.apk_listbox.curselection();
        if not sel:messagebox.showwarning("Error","Select APK first.",parent=self.root);return
        idx=sel[0];app=self.selected_apks_data[idx]
        imgPath=filedialog.askopenfilename(title=f"Select Image for {app.app_name_var.get()}",filetypes=(("Images","*.png *.jpg *.jpeg *.webp"),))
        if imgPath:app.update_image_selection(imgPath);self.selected_apk_image_label_var.set(f"Image: {os.path.basename(imgPath)}");self.display_icon_preview(imgPath)
    def calculate_apk_size_handler(self):
        sel=self.apk_listbox.curselection();
        if not sel:messagebox.showwarning("Error","Select APK first.",parent=self.root);return
        idx=sel[0];app=self.selected_apks_data[idx]
        if not app.is_url:
            try:size_b=os.path.getsize(app.apk_source);app.file_size_var.set(self.format_size(size_b));self.selected_apk_size_label_var.set(f"Size: {app.file_size_var.get()}")
            except OSError:messagebox.showerror("Error",f"Could not get size for local file: {app.apk_source}",parent=self.root);return
            return
        url=app.apk_source;self.selected_apk_size_label_var.set("Size: Calculating...");self.root.update_idletasks()
        threading.Thread(target=self._fetch_url_size_thread,args=(app,url),daemon=True).start()
    def _fetch_url_size_thread(self,app_entry,url):
        try:
            r_head=requests.head(url,timeout=10,allow_redirects=True);r_head.raise_for_status();cl=r_head.headers.get('content-length')
            if cl:size_b=int(cl);self.root.after(0,lambda:[app_entry.file_size_var.set(self.format_size(size_b)),self.selected_apk_size_label_var.set(f"Size: {app_entry.file_size_var.get()}"),messagebox.showinfo("Size","Size (headers): "+app_entry.file_size_var.get(),parent=self.root)]);return
            self.root.after(0,lambda:messagebox.showinfo("Info","Temp download for size...",parent=self.root))
            with tempfile.TemporaryDirectory()as t_dir:
                t_path=os.path.join(t_dir,"temp.apk")
                with requests.get(url, stream=True, timeout=60) as r:
                    r.raise_for_status() # בדוק שגיאות HTTP
                    with open(t_path, 'wb') as f: # ה-with השני בשורה חדשה ומוזחת
                        for chunk in r.iter_content(chunk_size=8192*4):
                            f.write(chunk)
                size_b=os.path.getsize(t_path)
                self.root.after(0,lambda:[app_entry.file_size_var.set(self.format_size(size_b)),self.selected_apk_size_label_var.set(f"Size: {app_entry.file_size_var.get()}"),messagebox.showinfo("Size","Size (download): "+app_entry.file_size_var.get(),parent=self.root)])
        except requests.exceptions.RequestException as e:self.root.after(0,lambda:[app_entry.file_size_var.set("Error"),self.selected_apk_size_label_var.set("Size: Error"),messagebox.showerror("Network Error",f"Fetch size error: {e}",parent=self.root)])
        except Exception as e:self.root.after(0,lambda:[app_entry.file_size_var.set("Error"),self.selected_apk_size_label_var.set("Size: Error"),messagebox.showerror("Error",f"Error: {e}",parent=self.root)])
    def format_size(self,sb):
        if sb<1024:return f"{sb} B"
        elif sb<1024*1024:return f"{sb/1024:.2f} KB"
        else:return f"{sb/(1024*1024):.2f} MB"
    def clear_apk_details_fields(self):
        self.selected_apk_source_label.config(text="Source: N/A");self.selected_apk_appname_entry.delete(0,tk.END);self.selected_apk_appname_entry.config(state=tk.DISABLED);self.selected_apk_package_entry.delete(0,tk.END);self.selected_apk_package_entry.config(state=tk.DISABLED);self.selected_apk_description_text.delete("1.0",tk.END);self.selected_apk_description_text.config(state=tk.DISABLED);self.selected_apk_image_label_var.set("Image: None");self.display_icon_preview(None);self.selected_apk_size_label_var.set("Size: N/A")
    def new_configuration(self):
        if messagebox.askyesno("New Config","Clear current settings and apps?",parent=self.root):
            self.selected_apks_data.clear();self.apk_listbox.delete(0,tk.END);self.clear_apk_details_fields()
            self.password_var.set("yossi6547");self.normal_install_var.set(False);self.use_root_var.set(True);self.root_method_var.set("pm_install");self.install_location_var.set("system/app");self.use_custom_storage_var.set(False);self.toggle_custom_path_entry();self.custom_storage_path_var.set("");self.send_feedback_var.set(True);self.about_text_widget.delete("1.0",tk.END);self.about_text_widget.insert("1.0","Default about text.");self.current_loaded_zip_path=None;log_message_gui(self.log_area,"New configuration.")
    def load_backup_zip_handler(self):
        zip_p=filedialog.askopenfilename(title="Open Backup ZIP",filetypes=(("ZIP", "*.zip"),));
        if not zip_p:return
        log_message_gui(self.log_area,f"\nLoading backup: {zip_p}");self.new_configuration()
        temp_sub=os.path.join(self.temp_extraction_dir,uuid.uuid4().hex);os.makedirs(temp_sub,exist_ok=True)
        try:
            with zipfile.ZipFile(zip_p,'r')as zf:
                manifest_d=json.loads(zf.read(MANIFEST_FILE_NAME).decode('utf-8'))
                self.password_var.set(manifest_d.get("settingsPassword","yossi6547"));self.normal_install_var.set(manifest_d.get("normalInstallMode",False));self.use_root_var.set(manifest_d.get("useRootInstall",True));self.root_method_var.set(manifest_d.get("rootInstallMethod","pm_install"));self.install_location_var.set(manifest_d.get("installLocation","system/app"));self.use_custom_storage_var.set(manifest_d.get("useCustomStoragePath",False));self.custom_storage_path_var.set(manifest_d.get("customStoragePath",""));self.toggle_custom_path_entry();self.send_feedback_var.set(manifest_d.get("enableSendFeedback",True));self.about_text_widget.delete("1.0",tk.END);self.about_text_widget.insert("1.0",manifest_d.get("aboutText","Default about text."))
                for app_d in manifest_d.get("appList",[]):
                    apk_fn_zip=app_d.get("apkFileName");apk_src=app_d.get("apkUrl");is_url=bool(apk_src);loc_apk_path=None
                    if not is_url_app and apk_fn_zip:
                        # Try to extract APK from ZIP
                        try:
                            apk_entry_zip = os.path.join(APKS_DIR_NAME, apk_fn_zip)
                            # zf.extractall(temp_sub, members=[m for m in zf.namelist() if m.startswith(APKS_DIR_NAME + '/')]) # Extracts all in apks/
                            # More specific extraction:
                            if apk_entry_zip in zf.namelist():
                                zf.extract(apk_entry_zip, temp_sub) # Extracts to temp_sub/apks/apk_fn_zip
                                ext_path = os.path.join(temp_sub, APKS_DIR_NAME, apk_fn_zip) # Correct path after extraction
                                if os.path.exists(ext_path):
                                    loc_apk_path = ext_path
                                    apk_src = loc_apk_path # Update apk_src to the local extracted path
                                    log_message_gui(self.log_area, f"  Extracted APK '{apk_fn_zip}' to {ext_path}")
                                else:
                                    log_message_gui(self.log_area, f"  WARNING: APK '{apk_fn_zip}' extracted, but not found at expected path: {ext_path}")
                            else:
                                log_message_gui(self.log_area, f"  WARNING: APK entry '{apk_entry_zip}' not found in ZIP.")

                        except KeyError: # Should be caught by 'in zf.namelist()' but good as a fallback
                            log_message_gui(self.log_area, f"  WARNING: APK '{apk_fn_zip}' listed in manifest but not found in ZIP's apks/ folder (KeyError).")
                        except Exception as e_extract_apk:
                            log_message_gui(self.log_area, f"  ERROR extracting APK '{apk_fn_zip}': {e_extract_apk}")
                    loc_img_path = None
                    img_fn_man = app_d.get("imageFileName")
                    if img_fn_man:
                        try:
                            img_entry_zip = os.path.join(IMAGES_DIR_NAME, img_fn_man)
                            if img_entry_zip in zf.namelist():
                                zf.extract(img_entry_zip, temp_sub) # Extracts to temp_sub/images/img_fn_man
                                ext_img_path = os.path.join(temp_sub, IMAGES_DIR_NAME, img_fn_man) # Correct path after extraction
                                if os.path.exists(ext_img_path):
                                    loc_img_path = ext_img_path
                                    log_message_gui(self.log_area, f"  Extracted Image '{img_fn_man}' to {ext_img_path}")
                                else:
                                    log_message_gui(self.log_area, f"  WARNING: Image '{img_fn_man}' extracted, but not found at expected path: {ext_img_path}")
                            else:
                                log_message_gui(self.log_area, f"  WARNING: Image entry '{img_entry_zip}' not found in ZIP.")
                        except KeyError:
                             log_message_gui(self.log_area, f"  WARNING: Image '{img_fn_man}' listed in manifest but not found in ZIP's images/ folder (KeyError).")
                        except Exception as e_extract_img:
                             log_message_gui(self.log_area, f"  ERROR extracting Image '{img_fn_man}': {e_extract_img}")
                    if apk_src:
                        app_e=AppEntry(apk_src,is_url,self.log_area,self.temp_icon_output_dir,app_d.get("name"),app_d.get("packageName"),app_d.get("description"),loc_img_path)
                        app_e.file_size_var.set(self.format_size(app_d.get("fileSize",0)));self.selected_apks_data.append(app_e);self.apk_listbox.insert(tk.END,app_e.get_display_name())
            self.current_loaded_zip_path=zip_p;messagebox.showinfo("Load Complete","Backup loaded.",parent=self.root)
            if self.selected_apks_data and self.apk_listbox.size()>0:self.apk_listbox.selection_clear(0,tk.END);self.apk_listbox.selection_set(0);self.on_apk_select_from_list(None)
        except Exception as e:messagebox.showerror("Load Error",f"Error loading backup: {e}",parent=self.root);traceback.print_exc()

    def create_backup_zip_handler(self, save_as=False):
        output_zip_path=self.current_loaded_zip_path
        if save_as or not output_zip_path:output_zip_path=filedialog.asksaveasfilename(title="Save Backup ZIP As",defaultextension=".zip",filetypes=(("ZIP", "*.zip"),))
        if not output_zip_path:messagebox.showwarning("Cancelled","Save location not selected.",parent=self.root);return
        self.update_selected_apk_details_from_fields()
        app_info_json=[e.to_dict()for e in self.selected_apks_data]
        config_data={"appList":app_info_json,"settingsPassword":self.password_var.get(),"normalInstallMode":self.normal_install_var.get(),"useRootInstall":self.use_root_var.get(),"rootInstallMethod":self.root_method_var.get(),"installLocation":self.install_location_var.get(),"useCustomStoragePath":self.use_custom_storage_var.get(),"customStoragePath":self.custom_storage_path_var.get()if self.use_custom_storage_var.get()else"","enableSendFeedback":self.send_feedback_var.get(),"aboutText":self.about_text_widget.get("1.0",tk.END).strip()}
        manifest_content=json.dumps(config_data,indent=4,ensure_ascii=False)
        try:
            with zipfile.ZipFile(output_zip_path,'w',zipfile.ZIP_DEFLATED)as zf:
                zf.writestr(MANIFEST_FILE_NAME,manifest_content)
                for app_e in self.selected_apks_data:
                    if not app_e.is_url and os.path.exists(app_e.apk_source):zf.write(app_e.apk_source,os.path.join(APKS_DIR_NAME,app_e.apk_file_name))
                    icon_p=app_e.local_image_path_var.get();icon_zip_name=app_e.image_file_name_in_zip_var.get()
                    if icon_p and icon_zip_name and os.path.exists(icon_p):zf.write(icon_p,os.path.join(IMAGES_DIR_NAME,icon_zip_name))
            self.current_loaded_zip_path=output_zip_path;messagebox.showinfo("Success",f"Backup created/updated at:\n{output_zip_path}",parent=self.root)
        except Exception as e:messagebox.showerror("Error",f"Failed to create backup: {e}",parent=self.root);traceback.print_exc()

    def load_gui_config(self): pass # Placeholder
    def save_gui_config(self): pass # Placeholder
    def on_closing(self):
        if hasattr(self,'temp_extraction_dir')and os.path.exists(self.temp_extraction_dir):
            try:shutil.rmtree(self.temp_extraction_dir);log_message_gui(self.log_area,"Temp extraction directory cleaned.")
            except Exception as e:log_message_gui(self.log_area,f"Error cleaning temp dir: {e}")
        self.root.destroy()

if __name__ == "__main__":
    if not PIL_AVAILABLE:
        try: root_temp = tk.Tk(); root_temp.withdraw(); messagebox.showwarning("Missing Library", "Pillow (PIL) library not found. Image previews will not work.\nPlease install it using: pip install Pillow"); root_temp.destroy()
        except tk.TclError: print("Pillow (PIL) library not found. Image previews will not work. Please install it using: pip install Pillow", file=sys.stderr)
    crash_log_filename = "backup_gui_crash_log.txt"
    try:
        main_root = tk.Tk()
        app = BackupCreatorApp(main_root)
        main_root.mainloop()
    except Exception as e_main:
        error_type=type(e_main).__name__;error_message=str(e_main);full_traceback=traceback.format_exc();error_details_for_popup=f"שגיאה: {error_type}"+(f" - {error_message.splitlines()[0]}" if error_message else "");print(f"CRITICAL ERROR: {error_type} - {error_message}\n{full_traceback}",file=sys.stderr)
        try:
            with open(crash_log_filename,"w",encoding="utf-8")as f:f.write(f"Unhandled exception:\nType: {error_type}\nMessage: {error_message}\n\nTraceback:\n{full_traceback}")
            try: error_gui_root=tk.Tk();error_gui_root.withdraw();messagebox.showerror("שגיאה קריטית",f"אירעה שגיאה והתוכנית נסגרת.\nפרטים בקובץ: {os.path.abspath(crash_log_filename)}\n\n{error_details_for_popup}",parent=None);error_gui_root.destroy()
            except tk.TclError:pass
        except Exception as e_log:print(f"Further error: {e_log}",file=sys.stderr)
        sys.exit(1)